/** MIT License - Copyright (c) 2021 Samarium */
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __require = typeof require !== "undefined" ? require : (x) => {
  throw new Error('Dynamic require of "' + x + '" is not supported');
};
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  default: () => QR
});
var import_constants = __toModule(require("./constants"));
var import_Module = __toModule(require("./Module"));
class QR {
  version;
  ecl;
  size;
  modules = [];
  constructor(version, ecl, modules) {
    this.version = version;
    this.ecl = ecl;
    this.size = version * 4 + 17;
    if (modules)
      this.modules = modules.map((row) => row.map((module2) => module2.copy()));
    else {
      for (let i = 0; i < this.size; i++) {
        const row = [];
        for (let j = 0; j < this.size; j++)
          row.push(new import_Module.Module());
        this.modules.push(row);
      }
    }
  }
  static getBitAt(bits, index) {
    return bits >>> index & 1;
  }
  getSize() {
    return this.size;
  }
  getModules() {
    return this.modules;
  }
  copy() {
    return new QR(this.version, this.ecl, this.modules);
  }
  toString() {
    return this.modules.map((row) => row.map((module2) => module2.getColor() ? 1 : 0).join(" ")).join("\n");
  }
  initialize() {
    this.drawTimingPatterns();
    this.drawFinderPatterns();
    this.drawSeparatorPatterns();
    this.drawAlignmentPatterns();
    this.drawDarkPattern();
    this.drawFormatPatterns();
    this.drawVersionInfoPatterns();
  }
  drawTimingPatterns() {
    for (let i = 0; i < this.size; i++) {
      const color = i % 2 == 0;
      this.modules[6][i] = new import_Module.FunctionalModule("TIMING", color);
      this.modules[i][6] = new import_Module.FunctionalModule("TIMING", color);
    }
  }
  drawFinderPatterns() {
    const pos = [[0, 0], [this.size - 7, 0], [0, this.size - 7]];
    for (const [x, y] of pos) {
      for (let i = 0; i < 7; i++) {
        for (let j = 0; j < 7; j++) {
          if (i == 0 || i == 6)
            this.modules[x + i][y + j] = new import_Module.FunctionalModule("FINDER", true);
          else if (i == 1 || i == 5)
            this.modules[x + i][y + j] = new import_Module.FunctionalModule("FINDER", !(j != 0 && j != 6));
          else
            this.modules[x + i][y + j] = new import_Module.FunctionalModule("FINDER", j != 1 && j != 5);
        }
      }
    }
  }
  drawSeparatorPatterns() {
    const pos1 = [[7, 0], [7, this.size - 8], [this.size - 8, 0]], pos2 = [[0, 7], [0, this.size - 8], [this.size - 8, 7]];
    for (const [x, y] of pos1) {
      for (let i = 0; i < 8; i++)
        this.modules[x][y + i] = new import_Module.FunctionalModule("SEPARATOR", false);
    }
    for (const [x, y] of pos2) {
      for (let i = 0; i < 8; i++)
        this.modules[x + i][y] = new import_Module.FunctionalModule("SEPARATOR", false);
    }
  }
  drawAlignmentPatterns() {
    if (this.version == 1)
      return;
    const p = import_constants.ALIGNMENT_POSITION[this.version];
    for (const x of p) {
      for (const y of p) {
        const module2 = this.modules[x][y];
        if (module2 instanceof import_Module.FunctionalModule && module2.getType() != "TIMING")
          continue;
        const pos = [x - 2, y - 2];
        for (let i = 0; i < 5; i++) {
          for (let j = 0; j < 5; j++) {
            const px = pos[0] + i, py = pos[1] + j;
            if (i == 0 || i == 4)
              this.modules[px][py] = new import_Module.FunctionalModule("ALIGNMENT", true);
            else if (i == 1 || i == 3)
              this.modules[px][py] = new import_Module.FunctionalModule("ALIGNMENT", !(j != 0 && j != 4));
            else
              this.modules[px][py] = new import_Module.FunctionalModule("ALIGNMENT", j != 1 && j != 3);
          }
        }
      }
    }
  }
  drawDarkPattern() {
    this.modules[this.version * 4 + 9][8] = new import_Module.FunctionalModule("Dark", true);
  }
  drawFormatPatterns(mask) {
    const bits = mask != void 0 ? import_constants.FORMAT_INFO[import_constants.ECL[this.ecl]][mask] : 0;
    for (let i = 0; i < 8; i++) {
      const color = QR.getBitAt(bits, i) != 0;
      this.modules[8][this.size - i - 1] = new import_Module.FunctionalModule("FORMAT_INFO", color);
      this.modules[i < 6 ? i : i + 1][8] = new import_Module.FunctionalModule("FORMAT_INFO", color);
    }
    for (let i = 0; i < 7; i++) {
      const color = QR.getBitAt(bits, i + 8) != 0;
      this.modules[8][i >= 1 ? 6 - i : 7 - i] = new import_Module.FunctionalModule("FORMAT_INFO", color);
      this.modules[this.size + i - 7][8] = new import_Module.FunctionalModule("FORMAT_INFO", color);
    }
  }
  drawVersionInfoPatterns() {
    if (this.version < 7)
      return;
    const bits = import_constants.VERSION_INFO[this.version - 7];
    for (let i = 0; i < 18; i++) {
      const x = Math.floor(i / 3), y = this.size - 11 + i % 3, color = QR.getBitAt(bits, i) != 0;
      this.modules[x][y] = new import_Module.FunctionalModule("VERSION_INFO", color);
      this.modules[y][x] = new import_Module.FunctionalModule("VERSION_INFO", color);
    }
  }
  generateDataPath() {
    const result = [];
    for (let col = this.size - 1; col > 0; col -= 2) {
      if (col == 6)
        col = 5;
      for (let row = 0; row < this.size; row++) {
        for (let i = 0; i < 2; i++) {
          const upward = (col + 1 & 2) == 0;
          const x = upward ? this.size - row - 1 : row;
          const y = col - i;
          if (this.modules[x][y] instanceof import_Module.FunctionalModule)
            continue;
          result.push([x, y]);
        }
      }
    }
    return result;
  }
  placeCodeword(data, path) {
    path.forEach(([x, y], index) => {
      const module2 = new import_Module.DataModule();
      if (index < data.length * 8) {
        const codeword = data[index >>> 3];
        module2.setColor(QR.getBitAt(codeword.getValue(), 7 - (index & 7)) != 0);
        index++;
      } else
        module2.setColor(false);
      this.modules[x][y] = module2;
    });
  }
  generateMask(which) {
    const result = new QR(this.version, this.ecl);
    for (let row = 0; row < this.size; row++) {
      for (let col = 0; col < this.size; col++) {
        let color;
        switch (which) {
          case 0:
            color = (row + col) % 2 == 0;
            break;
          case 1:
            color = row % 2 == 0;
            break;
          case 2:
            color = col % 3 == 0;
            break;
          case 3:
            color = (row + col) % 3 == 0;
            break;
          case 4:
            color = (Math.floor(row / 2) + Math.floor(col / 3)) % 2 == 0;
            break;
          case 5:
            color = row * col % 2 + row * col % 3 == 0;
            break;
          case 6:
            color = (row * col % 2 + row * col % 3) % 2 == 0;
            break;
          case 7:
            color = ((row + col) % 2 + row * col % 3) % 2 == 0;
            break;
          default:
            throw Error("Mask Value Error");
        }
        if (this.modules[row][col] instanceof import_Module.FunctionalModule)
          continue;
        result.modules[row][col] = new import_Module.MaskModule(color);
      }
    }
    return result;
  }
  generateAllMasks() {
    const result = [];
    for (let i = 0; i < 8; i++)
      result.push(this.generateMask(i));
    return result;
  }
  applyMask(mask) {
    for (let row = 0; row < this.size; row++) {
      for (let col = 0; col < this.size; col++) {
        const m = mask.modules[row][col];
        const original = this.modules[row][col];
        if (original instanceof import_Module.DataModule && m instanceof import_Module.MaskModule)
          original.setColor(original.getColor() != m.getColor());
      }
    }
  }
  computePenalty() {
    let dark = 0, p1 = 0, p2 = 0, p3 = 0;
    const sim1 = [true, false, true, true, true, false, true, false, false, false, false], sim2 = [false, false, false, false, true, false, true, true, true, false, true], r1 = (result, curr, index) => result && curr == sim1[index], r2 = (result, curr, index) => result && curr == sim2[index];
    for (let i = 0; i < this.size; i++) {
      let row_flag = true, col_flag = true, row_counter = 0, col_counter = 0;
      const row_queue = [], col_queue = [];
      for (let j = 0; j < this.size; j++) {
        const row_color = this.modules[i][j].getColor();
        row_queue.push(row_color);
        const col_color = this.modules[j][i].getColor();
        col_queue.push(col_color);
        if (row_color)
          dark++;
        if (row_color != row_flag) {
          row_flag = row_color;
          if (row_counter >= 5)
            p1 += import_constants.PENALTIES.S1 + row_counter - 5;
          row_counter = 1;
        } else
          row_counter++;
        if (col_color != col_flag) {
          col_flag = col_color;
          if (col_counter >= 5)
            p1 += import_constants.PENALTIES.S1 + col_counter - 5;
          col_counter = 1;
        } else
          col_counter++;
        if (j == this.size - 1) {
          if (row_counter >= 5)
            p1 += import_constants.PENALTIES.S1 + row_counter - 5;
          if (col_counter >= 5)
            p1 += import_constants.PENALTIES.S1 + col_counter - 5;
        } else if (i != this.size - 1) {
          const x = this.modules[i][j + 1].getColor(), y = this.modules[i + 1][j].getColor(), z = this.modules[i + 1][j + 1].getColor();
          if (row_color == x && y == z && x == y)
            p2 += import_constants.PENALTIES.S2;
        }
        if (row_queue.length == sim1.length) {
          const a2 = row_queue.reduce(r1, true), b2 = row_queue.reduce(r2, true), c = col_queue.reduce(r1, true), d = col_queue.reduce(r2, true);
          if (a2)
            p3 += import_constants.PENALTIES.S3;
          if (b2)
            p3 += import_constants.PENALTIES.S3;
          if (c)
            p3 += import_constants.PENALTIES.S3;
          if (d)
            p3 += import_constants.PENALTIES.S3;
          row_queue.splice(0, 1);
          col_queue.splice(0, 1);
        }
      }
    }
    const ratio = parseInt((dark / this.size ** 2 * 100).toFixed());
    let a = 0, b = 0;
    for (let i = 0; i < 5; i++) {
      const prev = ratio - i, next = ratio + i;
      if (prev % 5 == 0)
        a = prev;
      if (next % 5 == 0)
        b = next;
    }
    const p4 = Math.min(Math.abs(a - 50), Math.abs(b - 50)) * import_constants.PENALTIES.S4;
    return p1 + p2 + p3 + p4;
  }
  generateQuietRows() {
    const result = [];
    for (let i = 0; i < 4; i++) {
      const row = [];
      for (let j = 0; j < this.size + 8; j++)
        row.push(new import_Module.QuietModule());
      result.push(row);
    }
    return result;
  }
  extend() {
    const pre = this.generateQuietRows();
    for (let i = 0; i < this.size; i++) {
      const row = [];
      for (let j = -4; j < this.size + 4; j++)
        j < 0 || j > this.size - 1 ? row.push(new import_Module.QuietModule()) : row.push(this.modules[i][j]);
      pre.push(row);
    }
    this.modules = pre.concat(this.generateQuietRows());
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
//# sourceMappingURL=QR.js.map
