/** MIT License - Copyright (c) 2021 Samarium */
var __defProp = Object.defineProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __require = typeof require !== "undefined" ? require : (x) => {
  throw new Error('Dynamic require of "' + x + '" is not supported');
};
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
__export(exports, {
  default: () => RSECG
});
class RSECG {
  coefficients = [];
  constructor(degree) {
    if (degree < 1 || degree > 255)
      throw Error("Degree out of range");
    const coefs = this.coefficients;
    for (let i = 0; i < degree - 1; i++)
      coefs.push(0);
    coefs.push(1);
    let root = 1;
    for (let i = 0; i < degree; i++) {
      for (let j = 0; j < coefs.length; j++) {
        coefs[j] = RSECG.multiply(coefs[j], root);
        if (j + 1 < coefs.length)
          coefs[j] ^= coefs[j + 1];
      }
      root = RSECG.multiply(root, 2);
    }
  }
  static multiply(a, b) {
    if (a >>> 8 != 0 || b >>> 8 != 0)
      throw Error("Invalid bytes");
    let n = 0;
    for (let i = 7; i >= 0; i--) {
      n = n << 1 ^ (n >>> 7) * 285;
      n ^= (b >>> i & 1) * a;
    }
    if (n >>> 8 != 0)
      throw Error("Calculation error");
    return n;
  }
  getRemainder(data) {
    const result = this.coefficients.map(() => 0);
    for (const byte of data) {
      const f = byte ^ result.shift();
      result.push(0);
      this.coefficients.forEach((coefficient, index) => {
        result[index] ^= RSECG.multiply(coefficient, f);
      });
    }
    return result;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
//# sourceMappingURL=RSECG.js.map
