/** MIT License - Copyright (c) 2021 Samarium */
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __require = typeof require !== "undefined" ? require : (x) => {
  throw new Error('Dynamic require of "' + x + '" is not supported');
};
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  computeOptimalMask: () => computeOptimalMask,
  generateCodePoint: () => generateCodePoint,
  generateCodeword: () => generateCodeword,
  generateDataCodeword: () => generateDataCodeword,
  generateOptimalSegments: () => generateOptimalSegments,
  generateQR: () => generateQR,
  generateRawQR: () => generateRawQR,
  isColor: () => isColor
});
var constants = __toModule(require("./constants"));
var import_Segment = __toModule(require("./Segment"));
var import_CodePoint = __toModule(require("./CodePoint"));
var import_Codeword = __toModule(require("./Codeword"));
var import_RSECG = __toModule(require("./RSECG"));
var import_QR = __toModule(require("./QR"));
var import_emoji_regex = __toModule(require("emoji-regex"));
var import_encoding_japanese = __toModule(require("encoding-japanese"));
function isAlphanumeric(char) {
  return constants.ALPHANUMERIC.indexOf(char) != -1;
}
function isNumeric(char) {
  return constants.NUMERIC.indexOf(char) != -1;
}
function isKanji(char) {
  return (0, import_encoding_japanese.convert)(char, { to: "SJIS", type: "array" }).indexOf(63) == -1;
}
function isColor(str) {
  return /^((0x)?|#?)([0-9A-F]{8}|[0-9A-F]{6})$/ig.test(str);
}
function decToBin(n, length) {
  if (length < 0 || length > 31 || n >>> length != 0)
    throw Error("Value Error");
  const result = [];
  for (let i = length - 1; i >= 0; i--)
    result.push(n >>> i & 1);
  return result;
}
function getModeCharCount(mode, version) {
  return constants.MODE[mode].charCount[Math.floor((version + 7) / 17)];
}
function getNumEcCodeword(version, ecl) {
  return constants.EC_CODEWORD_PER_BLOCK[constants.ECL[ecl]][version];
}
function getNumEcBlocks(version, ecl) {
  return constants.NUM_EC_BLOCK[constants.ECL[ecl]][version];
}
function getCapacity(version, ecl) {
  return constants.CAPACITY[version][constants.ECL[ecl]];
}
function computeNumRawCodeword(version) {
  let result = (16 * version + 128) * version + 64;
  if (version >= 2) {
    const align = Math.floor(version / 7) + 2;
    result -= (25 * align - 10) * align - 55;
    if (version >= 7)
      result -= 36;
  }
  return Math.floor(result / 8);
}
function computeNumCodewords(segments, version) {
  let result = 0;
  for (const segment of segments) {
    const modeCharCountBits = getModeCharCount(segment.getMode(), version);
    if (segment.getCharLen() >= 1 << modeCharCountBits)
      throw Error("Cannot Encode");
    result += modeCharCountBits + segment.getData().length + 4;
  }
  return Math.ceil(result / 8);
}
function generateCodePoint(str) {
  const regex = (0, import_emoji_regex.default)(), points = [];
  if (str.search(regex) != -1) {
    const break_points = [], indices = [];
    let result = regex.exec(str);
    while (result) {
      indices.push(result.index);
      if (break_points[break_points.length - 1] != result.index)
        break_points.push(result.index);
      break_points.push(result.index + result[0].length);
      result = regex.exec(str);
    }
    break_points.push(str.length);
    for (let i = 0; i < break_points.length - 1; i++) {
      const sub = str.substring(break_points[i], break_points[i + 1]);
      if (indices.indexOf(break_points[i]) != -1)
        points.push(new import_CodePoint.default(sub));
      else
        for (const char of Array.from(sub))
          points.push(new import_CodePoint.default(char));
    }
  } else
    for (const char of Array.from(str))
      points.push(new import_CodePoint.default(char));
  return points;
}
function generateSegmentFromSingleMode(codePoints, mode) {
  const data = [];
  let len = codePoints.length;
  codePoints.forEach((point, index) => {
    let bits = "";
    switch (mode) {
      case "NUMERIC": {
        if (index % 3 == 0) {
          const n = Math.min(3, codePoints.length - index);
          const str = codePoints.slice(index, index + n).map((point2) => point2.getChar()).join("");
          bits = parseInt(str, 10).toString(2).padStart(n * 3 + 1, "0");
        }
        break;
      }
      case "ALPHANUMERIC": {
        let t = constants.ALPHANUMERIC.indexOf(point.getChar());
        if (index % 2 == 0) {
          const n = Math.min(2, codePoints.length - index);
          if (n == 2) {
            t = t * constants.ALPHANUMERIC.length + constants.ALPHANUMERIC.indexOf(codePoints[index + 1].getChar());
          }
          bits = t.toString(2).padStart(n * 5 + 1, "0");
        }
        break;
      }
      case "BYTE": {
        bits = point.getCode().map((code) => code.toString(2).toUpperCase().padStart(8, "0")).join("");
        len += point.getCode().length - 1;
        break;
      }
      case "KANJI": {
        const codes = (0, import_encoding_japanese.convert)(point.getChar(), { to: "SJIS", type: "array" }), rep = parseInt(codes.map((code) => code.toString(16)).join(""), 16), temp = constants.KANJI.A <= rep && rep <= constants.KANJI.B ? (rep - constants.KANJI.A).toString(16).padStart(4, "0") : constants.KANJI.C <= rep && rep <= constants.KANJI.D ? (rep - constants.KANJI.E).toString(16).padStart(4, "0") : "";
        if (temp.length == 0)
          throw Error("Kanji Encoding Error");
        const msb = parseInt(temp.substring(0, 2), 16), lsb = parseInt(temp.substring(2, 4), 16);
        bits = (msb * constants.KANJI.factor + lsb).toString(2).padStart(13, "0");
        break;
      }
      default:
        throw Error("Invalid Encoding Mode");
    }
    for (const bit of bits)
      data.push(parseInt(bit, 2));
  });
  return new import_Segment.default(mode, len, data);
}
function computeOptimalVersion(segments, version, ecl, forced) {
  if (forced) {
    const capacity = getCapacity(version, ecl), length = computeNumCodewords(segments, version);
    if (capacity < length)
      throw Error("Cannot Encode At This Version");
    return version;
  }
  let result = -1;
  for (let i = 1; i <= 40; i++) {
    const length = computeNumCodewords(segments, i);
    for (const key in constants.ECL) {
      if (key == ecl) {
        const capacity = getCapacity(i, ecl);
        if (length <= capacity && result == -1 && i >= version) {
          result = i;
          break;
        }
      }
    }
  }
  if (result == -1)
    throw Error("Cannot Encode");
  return result;
}
function generateSegmentsFromMultiModes(codePoints, modes) {
  const queue = [];
  let i = 1, mode = modes[0];
  const indices = [0], ms = [mode];
  while (i < codePoints.length) {
    if (modes[i] != mode) {
      indices.push(i);
      mode = modes[i];
      ms.push(mode);
    }
    i++;
  }
  indices.push(codePoints.length);
  i = 0;
  while (i < indices.length - 1) {
    queue.push([ms[i], codePoints.slice(indices[i], indices[i + 1])]);
    i++;
  }
  return queue.map((req) => generateSegmentFromSingleMode(req[1], req[0]));
}
function computeOptimalEncodingMode(codePoints, version) {
  const modes = ["BYTE", "ALPHANUMERIC", "NUMERIC", "KANJI"], header = modes.map((mode) => 4 + getModeCharCount(mode, version) * 6), charModes = [];
  let prev = header.slice();
  codePoints.forEach((point) => {
    const ms = modes.map(() => null), curr = modes.map(() => Infinity);
    ms[0] = "BYTE";
    curr[0] = prev[0] + 48 * point.getCode().length;
    if (isAlphanumeric(point.getChar())) {
      ms[1] = "ALPHANUMERIC";
      curr[1] = prev[1] + 33;
    }
    if (isNumeric(point.getChar())) {
      ms[2] = "NUMERIC";
      curr[2] = prev[2] + 20;
    }
    if (isKanji(point.getChar())) {
      ms[3] = "KANJI";
      curr[3] = prev[3] + 78;
    }
    modes.forEach((_, i) => modes.forEach((mode, j) => {
      const cost = Math.ceil(curr[j] / 6) * 6 + header[i];
      if (ms[j] && cost < curr[i]) {
        curr[i] = cost;
        ms[i] = mode;
      }
    }));
    charModes.push(ms);
    prev = curr;
  });
  let index = 0;
  modes.forEach((_, i) => prev[i] < prev[index] ? index = i : index);
  const result = [];
  for (let i = codePoints.length - 1; i >= 0; i--) {
    const mode = charModes[i][index];
    index = modes.indexOf(mode);
    result.push(mode);
  }
  return result.reverse();
}
function generateOptimalSegments(codePoints, version, ecl, forced) {
  if (forced) {
    const modes2 = computeOptimalEncodingMode(codePoints, version), segments2 = generateSegmentsFromMultiModes(codePoints, modes2);
    return [computeOptimalVersion(segments2, version, ecl, forced), segments2];
  }
  const versions = [1, 10, 27, 40];
  if (versions.indexOf(version) == -1) {
    versions.push(version);
    versions.sort((a, b) => a - b);
  }
  const index = versions.indexOf(version);
  versions.splice(0, index);
  for (let i = 0; i < versions.length - 1; i++) {
    const modes2 = computeOptimalEncodingMode(codePoints, versions[i]), segments2 = generateSegmentsFromMultiModes(codePoints, modes2), ver = computeOptimalVersion(segments2, versions[i], ecl, forced);
    if (versions[i] <= ver && ver <= versions[i + 1])
      return [ver, segments2];
  }
  const modes = computeOptimalEncodingMode(codePoints, versions[versions.length - 1]), segments = generateSegmentsFromMultiModes(codePoints, modes);
  return [computeOptimalVersion(segments, versions[versions.length - 1], ecl, forced), segments];
}
function generateDataCodeword(segments, version, ecl) {
  const result = [], bits = [], terminator = [0, 0, 0, 0], bitPad = [0, 0, 0, 0, 0, 0, 0];
  segments.forEach((segment) => {
    decToBin(constants.MODE[segment.getMode()].indicator, 4).forEach((bit) => bits.push(bit));
    decToBin(segment.getCharLen(), getModeCharCount(segment.getMode(), version)).forEach((bit) => bits.push(bit));
    segment.getData().forEach((bit) => bits.push(bit));
  });
  const capacity = getCapacity(version, ecl) * 8;
  terminator.slice(0, Math.min(4, capacity - bits.length)).forEach((bit) => bits.push(bit));
  bitPad.slice(0, (8 - bits.length % 8) % 8).forEach((bit) => bits.push(bit));
  const pad = [];
  for (let i = 0, n = (capacity - bits.length) / 8; i < n; i++) {
    if (i % 2 == 0)
      pad.push(1, 1, 1, 0, 1, 1, 0, 0);
    else
      pad.push(0, 0, 0, 1, 0, 0, 0, 1);
  }
  pad.forEach((bit) => bits.push(bit));
  for (let i = 0; i < bits.length; i += 8)
    result.push(new import_Codeword.DataCodeword(parseInt(bits.slice(i, i + 8).join(""), 2)));
  return result;
}
function splitData(data, version, ecl) {
  const num_blocks = getNumEcBlocks(version, ecl), num_ec = getNumEcCodeword(version, ecl), num_codewords = computeNumRawCodeword(version), num_short_block = num_blocks - num_codewords % num_blocks, short_block_length = Math.floor(num_codewords / num_blocks), result = [];
  for (let index = 0, off = 0; index < num_blocks; index++) {
    const end = off + short_block_length - num_ec + (index < num_short_block ? 0 : 1);
    const block = data.slice(off, end);
    block.forEach((dataCodeword, i) => {
      dataCodeword.setBlockIndex(index);
      dataCodeword.setIndex(i);
    });
    result.push(block);
    off = end;
  }
  return result;
}
function generateECCodeword(data_blocks, version, ecl) {
  const num_ec = getNumEcCodeword(version, ecl), short_block_length = data_blocks[0].length, ec = new import_RSECG.default(num_ec);
  let pre_inter_leave_index = 0;
  return data_blocks.map((block, index) => {
    for (const dataCodeword of block) {
      dataCodeword.setPreInterleaveIndex(pre_inter_leave_index);
      pre_inter_leave_index++;
    }
    const block_bytes = block.map((dataCodeWord) => dataCodeWord.getValue()), ec_bytes = ec.getRemainder(block_bytes);
    return ec_bytes.map((byte, i) => {
      const ecCodeword = new import_Codeword.ECCodeword(byte);
      ecCodeword.setPreInterleaveIndex(pre_inter_leave_index);
      pre_inter_leave_index++;
      ecCodeword.setBlockIndex(index);
      ecCodeword.setIndex(short_block_length + 1 + i);
      return ecCodeword;
    });
  });
}
function interleaveBlocks(data_blocks, ec_blocks) {
  const ec_block_length = ec_blocks[0].length, data_block_length = data_blocks[data_blocks.length - 1].length, result = [];
  for (let i = 0; i < data_block_length; i++) {
    data_blocks.forEach((block) => {
      if (i < block.length) {
        const codeword = block[i];
        codeword.setPostInterleaveIndex(result.length);
        result.push(codeword);
      }
    });
  }
  for (let i = 0; i < ec_block_length; i++) {
    ec_blocks.forEach((block) => {
      const codeword = block[i];
      codeword.setPostInterleaveIndex(result.length);
      result.push(codeword);
    });
  }
  return result;
}
function generateCodeword(data, version, ecl) {
  const data_blocks = splitData(data, version, ecl), ec_blocks = generateECCodeword(data_blocks, version, ecl);
  return interleaveBlocks(data_blocks, ec_blocks);
}
function generateRawQR(data, version, ecl) {
  const qr = new import_QR.default(version, ecl);
  qr.initialize();
  const path = qr.generateDataPath();
  qr.placeCodeword(data, path);
  return qr;
}
function computeOptimalMask(qr) {
  const masks = qr.generateAllMasks(), values = masks.map((mask, index) => {
    const copy = qr.copy();
    copy.applyMask(mask);
    copy.drawFormatPatterns(index);
    return copy.computePenalty();
  }), which = values.indexOf(Math.min(...values));
  return [which, masks[which]];
}
function generateQR(qr, mask) {
  qr.applyMask(mask[1]);
  qr.drawFormatPatterns(mask[0]);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  computeOptimalMask,
  generateCodePoint,
  generateCodeword,
  generateDataCodeword,
  generateOptimalSegments,
  generateQR,
  generateRawQR,
  isColor
});
//# sourceMappingURL=index.js.map
