/** MIT License - Copyright (c) 2021 Samarium */
import {
  generateCodePoint,
  generateOptimalSegments,
  generateCodeword,
  generateDataCodeword,
  generateRawQR,
  computeOptimalMask,
  generateQR
} from "./utils";
import { QuietModule } from "./utils/Module";
const defaultGeneratorOptions = {
  version: 1,
  ecl: "LOW",
  forced: false,
  mask: -1
};
const defaultRenderOptions = {
  c1: "#000000",
  c2: "#FFFFFF",
  size: 10
};
function generate(text, options = defaultGeneratorOptions) {
  const version = options.version, ecl = options.ecl, forced = options.forced, mask = options.mask, codePoints = generateCodePoint(text), [optimalVersion, segments] = generateOptimalSegments(codePoints, version, ecl, forced), codeword = generateCodeword(generateDataCodeword(segments, optimalVersion, ecl), optimalVersion, ecl), code = generateRawQR(codeword, optimalVersion, ecl), optimalMask = mask == -1 ? computeOptimalMask(code) : [mask, code.generateMask(mask)];
  generateQR(code, optimalMask);
  code.extend();
  return code;
}
function renderOnCanvas(code, id, options = defaultRenderOptions) {
  const output = document.createElement("canvas"), context = output.getContext("2d"), numBlock = code.getSize() + 8, size = options.size;
  if (!context)
    throw Error("Cannot get canvas context");
  output.id = id;
  output.width = numBlock * size;
  output.height = numBlock * size;
  const modules = code.getModules();
  for (let row = 0; row < numBlock; row++) {
    for (let col = 0; col < numBlock; col++) {
      const module = modules[row][col];
      context.fillStyle = module instanceof QuietModule ? "#FFFFFF" : module.getColor() ? options.c1 : options.c2;
      context.fillRect(col * size, row * size, size, size);
    }
  }
  context.stroke();
  return output;
}
export {
  generate,
  renderOnCanvas
};
//# sourceMappingURL=index.js.map
